@@ -48,6 +48,7 @@ constexpr const char DocumentRangeFormattingRequest::methodName[];
 constexpr const char DocumentOnTypeFormattingRequest::methodName[];
 constexpr const char RenameRequest::methodName[];
 constexpr const char SignatureHelpRequest::methodName[];
+constexpr const char SemanticHighlightNotification::methodName[];
 
 HoverContent LanguageServerProtocol::Hover::content() const
 {
@@ -78,7 +79,7 @@ Utils::optional<MarkupOrString> ParameterInformation::documentation() const
     return MarkupOrString(documentation);
 }
 
-bool SignatureHelp::isValid(QStringList *error) const
+bool SignatureHelp::isValid(ErrorHierarchy *error) const
 {
     return checkArray<SignatureInformation>(error, signaturesKey);
 }
@@ -117,12 +118,12 @@ void CodeActionParams::CodeActionContext::setOnly(const QList<CodeActionKind> &o
     insertArray(onlyKey, only);
 }
 
-bool CodeActionParams::CodeActionContext::isValid(QStringList *error) const
+bool CodeActionParams::CodeActionContext::isValid(ErrorHierarchy *error) const
 {
     return checkArray<Diagnostic>(error, diagnosticsKey);
 }
 
-bool CodeActionParams::isValid(QStringList *error) const
+bool CodeActionParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<Range>(error, rangeKey)
@@ -153,7 +154,7 @@ DocumentColorRequest::DocumentColorRequest(const DocumentColorParams &params)
     : Request(methodName, params)
 { }
 
-bool Color::isValid(QStringList *error) const
+bool Color::isValid(ErrorHierarchy *error) const
 {
     return check<int>(error, redKey)
             && check<int>(error, greenKey)
@@ -161,7 +162,7 @@ bool Color::isValid(QStringList *error) const
             && check<int>(error, alphaKey);
 }
 
-bool ColorPresentationParams::isValid(QStringList *error) const
+bool ColorPresentationParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<Color>(error, colorInfoKey)
@@ -200,7 +201,7 @@ void FormattingOptions::setProperty(const QString &key, const DocumentFormatting
         insert(key, *val);
 }
 
-bool FormattingOptions::isValid(QStringList *error) const
+bool FormattingOptions::isValid(ErrorHierarchy *error) const
 {
     return Utils::allOf(keys(), [this, &error](auto key){
         return (key == tabSizeKey && this->check<int>(error, key))
@@ -209,7 +210,7 @@ bool FormattingOptions::isValid(QStringList *error) const
     });
 }
 
-bool DocumentFormattingParams::isValid(QStringList *error) const
+bool DocumentFormattingParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<FormattingOptions>(error, optionsKey);
@@ -219,7 +220,7 @@ DocumentFormattingRequest::DocumentFormattingRequest(const DocumentFormattingPar
     : Request(methodName, params)
 { }
 
-bool DocumentRangeFormattingParams::isValid(QStringList *error) const
+bool DocumentRangeFormattingParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<Range>(error, rangeKey)
@@ -227,11 +228,11 @@ bool DocumentRangeFormattingParams::isValid(QStringList *error) const
 }
 
 DocumentRangeFormattingRequest::DocumentRangeFormattingRequest(
-        const DocumentFormattingParams &params)
+        const DocumentRangeFormattingParams &params)
     : Request(methodName, params)
 { }
 
-bool DocumentOnTypeFormattingParams::isValid(QStringList *error) const
+bool DocumentOnTypeFormattingParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<Position>(error, positionKey)
@@ -244,7 +245,7 @@ DocumentOnTypeFormattingRequest::DocumentOnTypeFormattingRequest(
     : Request(methodName, params)
 { }
 
-bool RenameParams::isValid(QStringList *error) const
+bool RenameParams::isValid(ErrorHierarchy *error) const
 {
     return check<TextDocumentIdentifier>(error, textDocumentKey)
             && check<Position>(error, positionKey)
@@ -368,7 +369,7 @@ HoverContent::HoverContent(const QJsonValue &value)
     }
 }
 
-bool HoverContent::isValid(QStringList *errorHierarchy) const
+bool HoverContent::isValid(ErrorHierarchy *errorHierarchy) const
 {
     if (Utils::holds_alternative<MarkedString>(*this)
             || Utils::holds_alternative<MarkupContent>(*this)
@@ -376,10 +377,10 @@ bool HoverContent::isValid(QStringList *errorHierarchy) const
         return true;
     }
     if (errorHierarchy) {
-        *errorHierarchy << QCoreApplication::translate(
-                               "LanguageServerProtocol::HoverContent",
-                               "HoverContent should be either MarkedString, "
-                               "MarkupContent, or QList<MarkedString>.");
+        errorHierarchy->setError(
+            QCoreApplication::translate("LanguageServerProtocol::HoverContent",
+                                        "HoverContent should be either MarkedString, "
+                                        "MarkupContent, or QList<MarkedString>."));
     }
     return false;
 }
@@ -394,7 +395,7 @@ DocumentFormattingProperty::DocumentFormattingProperty(const QJsonValue &value)
         *this = value.toString();
 }
 
-bool DocumentFormattingProperty::isValid(QStringList *error) const
+bool DocumentFormattingProperty::isValid(ErrorHierarchy *error) const
 {
     if (Utils::holds_alternative<bool>(*this)
             || Utils::holds_alternative<double>(*this)
@@ -402,9 +403,9 @@ bool DocumentFormattingProperty::isValid(QStringList *error) const
         return true;
     }
     if (error) {
-        *error << QCoreApplication::translate(
-                      "LanguageServerProtocol::MarkedString",
-                      "DocumentFormattingProperty should be either bool, double, or QString.");
+        error->setError(QCoreApplication::translate(
+            "LanguageServerProtocol::MarkedString",
+            "DocumentFormattingProperty should be either bool, double, or QString."));
     }
     return false;
 }
@@ -432,7 +433,7 @@ CodeActionResult::CodeActionResult(const QJsonValue &val)
     emplace<std::nullptr_t>(nullptr);
 }
 
-bool CodeAction::isValid(QStringList *error) const
+bool CodeAction::isValid(ErrorHierarchy *error) const
 {
     return check<QString>(error, titleKey)
            && checkOptional<CodeActionKind>(error, codeActionKindKey)
@@ -441,4 +442,59 @@ bool CodeAction::isValid(QStringList *error) const
            && checkOptional<Command>(error, commandKey);
 }
 
+Utils::optional<QList<SemanticHighlightToken>> SemanticHighlightingInformation::tokens() const
+{
+    QList<SemanticHighlightToken> resultTokens;
+
+    const QByteArray tokensByteArray = QByteArray::fromBase64(
+        typedValue<QString>(tokensKey).toLocal8Bit());
+    constexpr int tokensByteSize = 8;
+    int index = 0;
+    while (index + tokensByteSize <= tokensByteArray.size()) {
+        resultTokens << SemanticHighlightToken(tokensByteArray.mid(index, tokensByteSize));
+        index += tokensByteSize;
+    }
+    return Utils::make_optional(resultTokens);
+}
+
+void SemanticHighlightingInformation::setTokens(const QList<SemanticHighlightToken> &tokens)
+{
+    QByteArray byteArray;
+    byteArray.reserve(8 * tokens.size());
+    for (const SemanticHighlightToken &token : tokens)
+        token.appendToByteArray(byteArray);
+    insert(tokensKey, QString::fromLocal8Bit(byteArray.toBase64()));
+}
+
+SemanticHighlightToken::SemanticHighlightToken(const QByteArray &token)
+{
+    QTC_ASSERT(token.size() == 8, return );
+    character = ( quint32(token.at(0)) << 24
+                | quint32(token.at(1)) << 16
+                | quint32(token.at(2)) << 8
+                | quint32(token.at(3)));
+
+    length = quint16(token.at(4) << 8 | token.at(5));
+
+    scope = quint16(token.at(6) << 8 | token.at(7));
+}
+
+void SemanticHighlightToken::appendToByteArray(QByteArray &byteArray) const
+{
+    byteArray.append(char((character & 0xff000000) >> 24));
+    byteArray.append(char((character & 0x00ff0000) >> 16));
+    byteArray.append(char((character & 0x0000ff00) >> 8));
+    byteArray.append(char((character & 0x000000ff)));
+    byteArray.append(char((length & 0xff00) >> 8));
+    byteArray.append(char((length & 0x00ff)));
+    byteArray.append(char((scope & 0xff00) >> 8));
+    byteArray.append(char((scope & 0x00ff)));
+}
+
+bool SemanticHighlightingParams::isValid(ErrorHierarchy *error) const
+{
+    return check<VersionedTextDocumentIdentifier>(error, textDocumentKey)
+            && checkArray<SemanticHighlightingInformation>(error, linesKey);
+}
+
 } // namespace LanguageServerProtocol
