@@ -37,6 +37,7 @@ class QDomDocument;
 class QDomElement;
 class QComboBox;
 class QPushButton;
+class QHBoxLayout;
 class QLabel;
 class QLineEdit;
 class QListView;
@@ -115,9 +116,21 @@ protected:
     void focusInEvent(QFocusEvent *event) override;
 
 private:
+    void setMasterIcon();
+    void clearMasterIcon();
     void setLDPIIcon();
     void setMDPIIcon();
     void setHDPIIcon();
+    void clearLDPIIcon();
+    void clearMDPIIcon();
+    void clearHDPIIcon();
+    void createDPIButton(QHBoxLayout *layout,
+                         QWidget *parent,
+                         QToolButton *&button, const QSize &buttonSize,
+                         const QString &title, const QString &tooltip,
+                         QToolButton **clearButton = nullptr,
+                         QLabel **scaleWarningLabel = nullptr
+                         );
     void defaultPermissionOrFeatureCheckBoxClicked();
     void addPermission();
     void removePermission();
@@ -137,8 +150,12 @@ private:
                        int *errorLine, int *errorColumn);
     enum IconDPI { LowDPI, MediumDPI, HighDPI };
     QIcon icon(const QString &baseDir, IconDPI dpi);
-    QString iconPath(const QString &baseDir, IconDPI dpi);
+    QString iconPath(IconDPI dpi);
+    QSize iconSize(IconDPI dpi);
+    void updateIconPath(const QString &newPath, IconDPI dpi);
     void copyIcon(IconDPI dpi, const QString &baseDir, const QString &filePath);
+    void removeIcon(IconDPI dpi, const QString &baseDir);
+    void toggleIconScaleWarning(IconDPI dpi, bool visible);
 
     void updateInfoBar(const QString &errorMessage, int line, int column);
     void hideInfoBar();
@@ -172,10 +189,18 @@ private:
     QLineEdit *m_appNameLineEdit;
     QLineEdit *m_activityNameLineEdit;
     QComboBox *m_targetLineEdit;
+    QComboBox *m_styleExtractMethod;
+    QToolButton *m_masterIconButton;
     QToolButton *m_lIconButton;
+    QToolButton *m_lIconClearButton;
+    QLabel *m_lIconScaleWarningLabel;
     QToolButton *m_mIconButton;
+    QToolButton *m_mIconClearButton;
+    QLabel *m_mIconScaleWarningLabel;
     QToolButton *m_hIconButton;
-    QString m_lIconPath; // only set if the user changed the icon
+    QToolButton *m_hIconClearButton;
+    QLabel *m_hIconScaleWarningLabel;
+    QString m_lIconPath;
     QString m_mIconPath;
     QString m_hIconPath;
 
