@@ -32,8 +32,12 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/manhattanstyle.h>
 
+#include <debugger/analyzer/diagnosticlocation.h>
+
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/theme/theme.h>
+#include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QApplication>
@@ -46,65 +50,6 @@ using namespace Debugger;
 namespace ClangTools {
 namespace Internal {
 
-// A header view that puts a check box in front of a given column.
-class HeaderWithCheckBoxInColumn : public QHeaderView
-{
-    Q_OBJECT
-
-public:
-    HeaderWithCheckBoxInColumn(Qt::Orientation orientation,
-                               int column = 0,
-                               QWidget *parent = nullptr)
-        : QHeaderView(orientation, parent)
-        , m_column(column)
-    {
-        setDefaultAlignment(Qt::AlignLeft);
-    }
-
-    void setState(QFlags<QStyle::StateFlag> newState) { state = newState; }
-
-protected:
-    void paintSection(QPainter *painter, const QRect &rect, int logicalIndex) const override
-    {
-        painter->save();
-        QHeaderView::paintSection(painter, rect, logicalIndex);
-        painter->restore();
-        if (logicalIndex == m_column) {
-            QStyleOptionButton option;
-            const int side = sizeHint().height();
-            option.rect = QRect(rect.left() + 1, 1, side - 3, side - 3);
-            option.state = state;
-            painter->save();
-            const int shift = side - 2;
-            painter->translate(QPoint(shift, 0));
-            QHeaderView::paintSection(painter, rect.adjusted(0, 0, -shift, 0), logicalIndex);
-            painter->restore();
-            style()->drawPrimitive(QStyle::PE_IndicatorCheckBox, &option, painter);
-        }
-    }
-
-    void mouseReleaseEvent(QMouseEvent *event) override
-    {
-        const int x = event->localPos().x();
-        const int columnX = sectionPosition(m_column);
-        const bool isWithinCheckBox = x > columnX && x < columnX + sizeHint().height() - 3;
-        if (isWithinCheckBox) {
-            state = (state != QStyle::State_On) ? QStyle::State_On : QStyle::State_Off;
-            viewport()->update();
-            emit checkBoxClicked(state == QStyle::State_On);
-            return; // Avoid changing sort order
-        }
-        QHeaderView::mouseReleaseEvent(event);
-    }
-
-signals:
-    void checkBoxClicked(bool checked);
-
-private:
-    const int m_column = 0;
-    QFlags<QStyle::StateFlag> state = QStyle::State_Off;
-};
-
 static QString getBaseStyleName()
 {
     QStyle *style = QApplication::style();
@@ -154,8 +99,9 @@ class DiagnosticViewDelegate : public QStyledItemDelegate
     Q_OBJECT
 
 public:
-    DiagnosticViewDelegate(DiagnosticViewStyle *style)
-        : m_style(style)
+    DiagnosticViewDelegate(DiagnosticViewStyle *style, QObject *parent)
+        : QStyledItemDelegate(parent)
+        ,  m_style(style)
     {}
 
     void paint(QPainter *painter,
@@ -178,18 +124,71 @@ private:
 DiagnosticView::DiagnosticView(QWidget *parent)
     : Debugger::DetailedErrorView(parent)
     , m_style(new DiagnosticViewStyle)
-    , m_delegate(new DiagnosticViewDelegate(m_style.get()))
+    , m_delegate(new DiagnosticViewDelegate(m_style, this))
 {
+    header()->hide();
+
+    const QIcon filterIcon
+        = Utils::Icon({{":/utils/images/filtericon.png", Utils::Theme::IconsBaseColor}}).icon();
+
+    m_showFilter = new QAction(tr("Filter..."), this);
+    m_showFilter->setIcon(filterIcon);
+    connect(m_showFilter, &QAction::triggered,
+            this, &DiagnosticView::showFilter);
+    m_clearFilter = new QAction(tr("Clear Filter"), this);
+    m_clearFilter->setIcon(filterIcon);
+    connect(m_clearFilter, &QAction::triggered,
+            this, &DiagnosticView::clearFilter);
+    m_filterForCurrentKind = new QAction(tr("Filter for This Diagnostic Kind"), this);
+    m_filterForCurrentKind->setIcon(filterIcon);
+    connect(m_filterForCurrentKind, &QAction::triggered,
+            this, &DiagnosticView::filterForCurrentKind);
+    m_filterOutCurrentKind = new QAction(tr("Filter out This Diagnostic Kind"), this);
+    m_filterOutCurrentKind->setIcon(filterIcon);
+    connect(m_filterOutCurrentKind, &QAction::triggered,
+            this, &DiagnosticView::filterOutCurrentKind);
+
+    m_separator = new QAction(this);
+    m_separator->setSeparator(true);
+
+    m_separator2 = new QAction(this);
+    m_separator2->setSeparator(true);
+
+    m_help = new QAction(tr("Web Page"), this);
+    m_help->setIcon(Utils::Icons::INFO.icon());
+    connect(m_help, &QAction::triggered,
+            this, &DiagnosticView::showHelp);
+
     m_suppressAction = new QAction(tr("Suppress This Diagnostic"), this);
     connect(m_suppressAction, &QAction::triggered,
             this, &DiagnosticView::suppressCurrentDiagnostic);
+
     installEventFilter(this);
 
-    setStyle(m_style.get());
-    setItemDelegate(m_delegate.get());
+    setStyle(m_style);
+    setItemDelegate(m_delegate);
 }
 
-DiagnosticView::~DiagnosticView() = default;
+void DiagnosticView::scheduleAllFixits(bool schedule)
+{
+    const auto proxyModel = static_cast<QSortFilterProxyModel *>(model());
+    for (int i = 0, count = proxyModel->rowCount(); i < count; ++i) {
+        const QModelIndex filePathItemIndex = proxyModel->index(i, 0);
+        for (int j = 0, count = proxyModel->rowCount(filePathItemIndex); j < count; ++j) {
+            const QModelIndex proxyIndex = proxyModel->index(j, 0, filePathItemIndex);
+            const QModelIndex diagnosticItemIndex = proxyModel->mapToSource(proxyIndex);
+            auto item = static_cast<DiagnosticItem *>(diagnosticItemIndex.internalPointer());
+            item->setData(DiagnosticView::DiagnosticColumn,
+                          schedule ? Qt::Checked : Qt::Unchecked,
+                          Qt::CheckStateRole);
+        }
+    }
+}
+
+DiagnosticView::~DiagnosticView()
+{
+    delete m_style;
+}
 
 void DiagnosticView::suppressCurrentDiagnostic()
 {
@@ -210,9 +209,8 @@ void DiagnosticView::suppressCurrentDiagnostic()
                 = filePath.relativeChildPath(project->projectDirectory());
         if (!relativeFilePath.isEmpty())
             filePath = relativeFilePath;
-        const SuppressedDiagnostic supDiag(filePath, diag.description, diag.issueContextKind,
-                                           diag.issueContext, diag.explainingSteps.count());
-        ClangToolsProjectSettingsManager::getSettings(project)->addSuppressedDiagnostic(supDiag);
+        const SuppressedDiagnostic supDiag(filePath, diag.description, diag.explainingSteps.count());
+        ClangToolsProjectSettings::getSettings(project)->addSuppressedDiagnostic(supDiag);
     } else {
         filterModel->addSuppressedDiagnostic(SuppressedDiagnostic(diag));
     }
@@ -268,7 +266,26 @@ QModelIndex DiagnosticView::getTopLevelIndex(const QModelIndex &index, Direction
 
 QList<QAction *> DiagnosticView::customActions() const
 {
-    return {m_suppressAction};
+    const QModelIndex currentIndex = selectionModel()->currentIndex();
+
+    const bool isDiagnosticItem = currentIndex.parent().isValid();
+    const QString docUrl
+        = model()->data(currentIndex, ClangToolsDiagnosticModel::DocumentationUrlRole).toString();
+    m_help->setEnabled(isDiagnosticItem && !docUrl.isEmpty());
+    m_filterForCurrentKind->setEnabled(isDiagnosticItem);
+    m_filterOutCurrentKind->setEnabled(isDiagnosticItem);
+    m_suppressAction->setEnabled(isDiagnosticItem);
+
+    return {
+        m_help,
+        m_separator,
+        m_showFilter,
+        m_clearFilter,
+        m_filterForCurrentKind,
+        m_filterOutCurrentKind,
+        m_separator2,
+        m_suppressAction,
+    };
 }
 
 bool DiagnosticView::eventFilter(QObject *watched, QEvent *event)
@@ -284,7 +301,7 @@ bool DiagnosticView::eventFilter(QObject *watched, QEvent *event)
         return true;
     }
     default:
-        return QObject::eventFilter(watched, event);
+        return QAbstractItemView::eventFilter(watched, event);
     }
 }
 
@@ -294,17 +311,6 @@ void DiagnosticView::mouseDoubleClickEvent(QMouseEvent *event)
     Debugger::DetailedErrorView::mouseDoubleClickEvent(event);
 }
 
-void DiagnosticView::setSelectedFixItsCount(int fixItsCount)
-{
-    if (m_ignoreSetSelectedFixItsCount)
-        return;
-    auto checkBoxHeader = static_cast<HeaderWithCheckBoxInColumn *>(header());
-    checkBoxHeader->setState(fixItsCount
-                                 ? (QStyle::State_NoChange | QStyle::State_On | QStyle::State_Off)
-                                 : QStyle::State_Off);
-    checkBoxHeader->viewport()->update();
-}
-
 void DiagnosticView::openEditorForCurrentIndex()
 {
     const QVariant v = model()->data(currentIndex(), Debugger::DetailedErrorView::LocationRole);
@@ -313,36 +319,6 @@ void DiagnosticView::openEditorForCurrentIndex()
         Core::EditorManager::openEditorAt(loc.filePath, loc.line, loc.column - 1);
 }
 
-void DiagnosticView::setModel(QAbstractItemModel *theProxyModel)
-{
-    const auto proxyModel = static_cast<QSortFilterProxyModel *>(theProxyModel);
-    Debugger::DetailedErrorView::setModel(proxyModel);
-
-    auto *header = new HeaderWithCheckBoxInColumn(Qt::Horizontal,
-                                                  DiagnosticView::DiagnosticColumn,
-                                                  this);
-    connect(header, &HeaderWithCheckBoxInColumn::checkBoxClicked, this, [=](bool checked) {
-        m_ignoreSetSelectedFixItsCount = true;
-        for (int i = 0, count = proxyModel->rowCount(); i < count; ++i) {
-            const QModelIndex filePathItemIndex = proxyModel->index(i, 0);
-            for (int j = 0, count = proxyModel->rowCount(filePathItemIndex); j < count; ++j) {
-                const QModelIndex proxyIndex = proxyModel->index(j, 0, filePathItemIndex);
-                const QModelIndex diagnosticItemIndex = proxyModel->mapToSource(proxyIndex);
-                auto item = static_cast<DiagnosticItem *>(diagnosticItemIndex.internalPointer());
-                item->setData(DiagnosticView::DiagnosticColumn,
-                              checked ? Qt::Checked : Qt::Unchecked,
-                              Qt::CheckStateRole);
-            }
-        }
-        m_ignoreSetSelectedFixItsCount = false;
-    });
-    setHeader(header);
-    header->setSectionResizeMode(DiagnosticView::DiagnosticColumn, QHeaderView::Stretch);
-    const int columnWidth = header->sectionSizeHint(DiagnosticView::DiagnosticColumn);
-    const int checkboxWidth = header->height();
-    header->setMinimumSectionSize(columnWidth + 1.2 * checkboxWidth);
-}
-
 } // namespace Internal
 } // namespace ClangTools
 
