@@ -25,48 +25,68 @@
 
 #pragma once
 
+#include "cmakeconfigitem.h"
+
 #include <QAbstractTableModel>
+#include <utils/treemodel.h>
 
 namespace CMakeProjectManager {
 
-class ConfigModel : public QAbstractTableModel
+namespace Internal {  class ConfigModelTreeItem; }
+
+class ConfigModel : public Utils::TreeModel<>
 {
     Q_OBJECT
 
 public:
+    enum Roles {
+        ItemIsAdvancedRole = Qt::UserRole,
+    };
+
     class DataItem {
     public:
         enum Type { BOOLEAN, FILE, DIRECTORY, STRING, UNKNOWN};
 
         QString key;
         Type type = STRING;
+        bool isHidden = false;
         bool isAdvanced = false;
+        bool inCMakeCache = false;
+        bool isUnset = false;
         QString value;
         QString description;
+        QStringList values;
     };
 
     explicit ConfigModel(QObject *parent = nullptr);
+    ~ConfigModel() override;
 
-    // QAbstractItemModel interface
-    int rowCount(const QModelIndex &parent) const override;
-    int columnCount(const QModelIndex &parent) const override;
-    Qt::ItemFlags flags(const QModelIndex &index) const override;
-    QVariant data(const QModelIndex &index, int role) const override;
-    bool setData(const QModelIndex &index, const QVariant &value, int role) override;
-    QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
+    QVariant data(const QModelIndex &idx, int role) const final;
 
     void appendConfiguration(const QString &key,
                              const QString &value = QString(),
                              const DataItem::Type type = DataItem::UNKNOWN,
-                             const QString &description = QString());
+                             const QString &description = QString(),
+                             const QStringList &values = QStringList());
+    void setConfiguration(const CMakeConfig &config);
     void setConfiguration(const QList<DataItem> &config);
+    void setConfigurationFromKit(const QHash<QString, QString> &kitConfig);
+    void setConfigurationForCMake(const QHash<QString, QString> &config);
     void flush();
     void resetAllChanges();
 
     bool hasChanges() const;
     bool hasCMakeChanges() const;
 
-    QList<DataItem> configurationChanges() const;
+    bool canForceTo(const QModelIndex &idx, const DataItem::Type type) const;
+    void forceTo(const QModelIndex &idx, const DataItem::Type type);
+
+    void toggleUnsetFlag(const QModelIndex &idx);
+
+    static DataItem dataItemFromIndex(const QModelIndex &idx);
+
+    QList<DataItem> configurationForCMake() const;
+
 
 private:
     class InternalDataItem : public DataItem
@@ -75,15 +95,42 @@ private:
         InternalDataItem(const DataItem &item);
         InternalDataItem(const InternalDataItem &item) = default;
 
-        bool isUserChanged;
-        bool isUserNew;
-        bool isCMakeChanged;
+        QString toolTip() const;
+        QString currentValue() const;
+
+        bool isUserChanged = false;
+        bool isUserNew = false;
+        bool isCMakeChanged = false;
         QString newValue;
+        QString kitValue;
     };
 
-    InternalDataItem &itemAtRow(int row);
-    const InternalDataItem &itemAtRow(int row) const;
+    void generateTree();
+
+    void setConfiguration(const QList<InternalDataItem> &config);
     QList<InternalDataItem> m_configuration;
+    QHash<QString, QString> m_kitConfiguration;
+
+    friend class Internal::ConfigModelTreeItem;
 };
 
+namespace Internal {
+
+class ConfigModelTreeItem  : public Utils::TreeItem
+{
+public:
+    ConfigModelTreeItem(ConfigModel::InternalDataItem *di = nullptr) : dataItem(di) {}
+    ~ConfigModelTreeItem() override;
+
+    QVariant data(int column, int role) const final;
+    bool setData(int column, const QVariant &data, int role) final;
+    Qt::ItemFlags flags(int column) const final;
+
+    QString toolTip() const;
+    QString currentValue() const;
+
+    ConfigModel::InternalDataItem *dataItem;
+};
+
+} // namespace Internal
 } // namespace CMakeProjectManager
