@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,42 +9,45 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "editorview.h"
-#include "editormanager.h"
-#include "documentmodel.h"
 
+#include "editormanager.h"
+#include "editormanager_p.h"
+#include "documentmodel.h"
+#include "documentmodel_p.h"
+
+#include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/editortoolbar.h>
-#include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/infobar.h>
+#include <coreplugin/locator/locatorconstants.h>
 #include <coreplugin/minisplitter.h>
 #include <coreplugin/editormanager/ieditor.h>
-
 #include <coreplugin/findplaceholder.h>
 #include <utils/qtcassert.h>
+#include <utils/theme/theme.h>
+#include <utils/utilsicons.h>
 
 #include <QDebug>
 
+#include <QFileInfo>
 #include <QHBoxLayout>
 #include <QLabel>
+#include <QMenu>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QStackedWidget>
@@ -54,32 +57,38 @@
 
 using namespace Core;
 using namespace Core::Internal;
-
+using namespace Utils;
 
 // ================EditorView====================
 
 EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
     QWidget(parent),
     m_parentSplitterOrView(parentSplitterOrView),
-    m_toolBar(EditorManager::createToolBar(this)),
+    m_toolBar(new EditorToolBar(this)),
     m_container(new QStackedWidget(this)),
     m_infoBarDisplay(new InfoBarDisplay(this)),
     m_statusHLine(new QFrame(this)),
-    m_statusWidget(new QFrame(this)),
-    m_currentNavigationHistoryPosition(0)
+    m_statusWidget(new QFrame(this))
 {
-    QVBoxLayout *tl = new QVBoxLayout(this);
+    auto tl = new QVBoxLayout(this);
     tl->setSpacing(0);
-    tl->setMargin(0);
+    tl->setContentsMargins(0, 0, 0, 0);
     {
-        connect(m_toolBar, SIGNAL(goBackClicked()), this, SLOT(goBackInNavigationHistory()));
-        connect(m_toolBar, SIGNAL(goForwardClicked()), this, SLOT(goForwardInNavigationHistory()));
-        connect(m_toolBar, SIGNAL(closeClicked()), this, SLOT(closeView()));
-        connect(m_toolBar, SIGNAL(listSelectionActivated(int)), this, SLOT(listSelectionActivated(int)));
-        connect(m_toolBar, SIGNAL(horizontalSplitClicked()), this, SLOT(splitHorizontally()));
-        connect(m_toolBar, SIGNAL(verticalSplitClicked()), this, SLOT(splitVertically()));
-        connect(m_toolBar, SIGNAL(splitNewWindowClicked()), this, SLOT(splitNewWindow()));
-        connect(m_toolBar, SIGNAL(closeSplitClicked()), this, SLOT(closeSplit()));
+        connect(m_toolBar, &EditorToolBar::goBackClicked,
+                this, &EditorView::goBackInNavigationHistory);
+        connect(m_toolBar, &EditorToolBar::goForwardClicked,
+                this, &EditorView::goForwardInNavigationHistory);
+        connect(m_toolBar, &EditorToolBar::closeClicked, this, &EditorView::closeCurrentEditor);
+        connect(m_toolBar, &EditorToolBar::listSelectionActivated,
+                this, &EditorView::listSelectionActivated);
+        connect(m_toolBar, &EditorToolBar::currentDocumentMoved,
+                this, &EditorView::closeCurrentEditor);
+        connect(m_toolBar, &EditorToolBar::horizontalSplitClicked,
+                this, &EditorView::splitHorizontally);
+        connect(m_toolBar, &EditorToolBar::verticalSplitClicked, this, &EditorView::splitVertically);
+        connect(m_toolBar, &EditorToolBar::splitNewWindowClicked, this, &EditorView::splitNewWindow);
+        connect(m_toolBar, &EditorToolBar::closeSplitClicked, this, &EditorView::closeSplit);
+        m_toolBar->setMenuProvider([this](QMenu *menu) { fillListContextMenu(menu); });
         tl->addWidget(m_toolBar);
     }
 
@@ -96,7 +105,7 @@ EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
         m_statusWidget->setLineWidth(0);
         m_statusWidget->setAutoFillBackground(true);
 
-        QHBoxLayout *hbox = new QHBoxLayout(m_statusWidget);
+        auto hbox = new QHBoxLayout(m_statusWidget);
         hbox->setContentsMargins(1, 0, 1, 1);
         m_statusWidgetLabel = new QLabel;
         m_statusWidgetLabel->setContentsMargins(3, 0, 3, 0);
@@ -114,16 +123,36 @@ EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
     }
 
     // for the case of no document selected
-    QWidget *empty = new QWidget;
+    auto empty = new QWidget;
+    empty->hide();
+    auto emptyLayout = new QGridLayout(empty);
+    empty->setLayout(emptyLayout);
+    m_emptyViewLabel = new QLabel;
+    connect(EditorManagerPrivate::instance(), &EditorManagerPrivate::placeholderTextChanged,
+            m_emptyViewLabel, &QLabel::setText);
+    m_emptyViewLabel->setText(EditorManagerPrivate::placeholderText());
+    emptyLayout->addWidget(m_emptyViewLabel);
     m_container->addWidget(empty);
-    m_widgetEditorMap.insert(empty, 0);
+    m_widgetEditorMap.insert(empty, nullptr);
+
+    auto dropSupport = new DropSupport(this, [this](QDropEvent *event, DropSupport *dropSupport) -> bool {
+        // do not accept move events except from other editor views (i.e. their tool bars)
+        // otherwise e.g. item views that support moving items within themselves would
+        // also "move" the item into the editor view, i.e. the item would be removed from the
+        // item view
+        if (!qobject_cast<EditorToolBar*>(event->source()))
+            event->setDropAction(Qt::CopyAction);
+        if (event->type() == QDropEvent::DragEnter && !dropSupport->isFileDrop(event))
+            return false; // do not accept drops without files
+        return event->source() != m_toolBar; // do not accept drops on ourselves
+    });
+    connect(dropSupport, &DropSupport::filesDropped,
+            this, &EditorView::openDroppedFiles);
 
     updateNavigatorActions();
 }
 
-EditorView::~EditorView()
-{
-}
+EditorView::~EditorView() = default;
 
 SplitterOrView *EditorView::parentSplitterOrView() const
 {
@@ -133,16 +162,16 @@ SplitterOrView *EditorView::parentSplitterOrView() const
 EditorView *EditorView::findNextView()
 {
     SplitterOrView *current = parentSplitterOrView();
-    QTC_ASSERT(current, return 0);
+    QTC_ASSERT(current, return nullptr);
     SplitterOrView *parent = current->findParentSplitter();
     while (parent) {
         QSplitter *splitter = parent->splitter();
-        QTC_ASSERT(splitter, return 0);
-        QTC_ASSERT(splitter->count() == 2, return 0);
+        QTC_ASSERT(splitter, return nullptr);
+        QTC_ASSERT(splitter->count() == 2, return nullptr);
         // is current the first child? then the next view is the first one in current's sibling
         if (splitter->widget(0) == current) {
-            SplitterOrView *second = qobject_cast<SplitterOrView *>(splitter->widget(1));
-            QTC_ASSERT(second, return 0);
+            auto second = qobject_cast<SplitterOrView *>(splitter->widget(1));
+            QTC_ASSERT(second, return nullptr);
             return second->findFirstView();
         }
         // otherwise go up the hierarchy
@@ -150,28 +179,51 @@ EditorView *EditorView::findNextView()
         parent = current->findParentSplitter();
     }
     // current has no parent, so we are at the top and there is no "next" view
-    return 0;
+    return nullptr;
 }
 
-void EditorView::closeView()
+EditorView *EditorView::findPreviousView()
+{
+    SplitterOrView *current = parentSplitterOrView();
+    QTC_ASSERT(current, return nullptr);
+    SplitterOrView *parent = current->findParentSplitter();
+    while (parent) {
+        QSplitter *splitter = parent->splitter();
+        QTC_ASSERT(splitter, return nullptr);
+        QTC_ASSERT(splitter->count() == 2, return nullptr);
+        // is current the last child? then the previous view is the first child in current's sibling
+        if (splitter->widget(1) == current) {
+            auto first = qobject_cast<SplitterOrView *>(splitter->widget(0));
+            QTC_ASSERT(first, return nullptr);
+            return first->findFirstView();
+        }
+        // otherwise go up the hierarchy
+        current = parent;
+        parent = current->findParentSplitter();
+    }
+    // current has no parent, so we are at the top and there is no "previous" view
+    return nullptr;
+}
+
+void EditorView::closeCurrentEditor()
 {
     IEditor *editor = currentEditor();
     if (editor)
-       EditorManager::closeEditor(editor);
+       EditorManagerPrivate::closeEditorOrDocument(editor);
 }
 
 void EditorView::showEditorStatusBar(const QString &id,
                                      const QString &infoText,
                                      const QString &buttonText,
-                                     QObject *object, const char *member)
+                                     QObject *object, const std::function<void()> &function)
 {
     m_statusWidgetId = id;
     m_statusWidgetLabel->setText(infoText);
     m_statusWidgetButton->setText(buttonText);
     m_statusWidgetButton->setToolTip(buttonText);
     m_statusWidgetButton->disconnect();
-    if (object && member)
-        connect(m_statusWidgetButton, SIGNAL(clicked()), object, member);
+    if (object && function)
+        connect(m_statusWidgetButton, &QToolButton::clicked, object, function);
     m_statusWidget->setVisible(true);
     m_statusHLine->setVisible(true);
     //m_editorForInfoWidget = currentEditor();
@@ -208,16 +260,16 @@ void EditorView::updateEditorHistory(IEditor *editor, QList<EditLocation> &histo
 
     EditLocation location;
     location.document = document;
-    location.fileName = document->filePath();
-    location.id = editor->id();
+    location.fileName = document->filePath().toString();
+    location.id = document->id();
     location.state = QVariant(state);
 
     for (int i = 0; i < history.size(); ++i) {
-        if (history.at(i).document == 0
-            || history.at(i).document == document
-            ){
+        const EditLocation &item = history.at(i);
+        if (item.document == document
+                || (!item.document
+                    && !DocumentModel::indexOfFilePath(FilePath::fromString(item.fileName)))) {
             history.removeAt(i--);
-            continue;
         }
     }
     history.prepend(location);
@@ -225,7 +277,7 @@ void EditorView::updateEditorHistory(IEditor *editor, QList<EditLocation> &histo
 
 void EditorView::paintEvent(QPaintEvent *)
 {
-    EditorView *editorView = EditorManager::currentEditorView();
+    EditorView *editorView = EditorManagerPrivate::currentEditorView();
     if (editorView != this)
         return;
 
@@ -234,11 +286,17 @@ void EditorView::paintEvent(QPaintEvent *)
 
     // Discreet indication where an editor would be if there is none
     QPainter painter(this);
-    painter.setRenderHint(QPainter::Antialiasing, true);
-    painter.setPen(Qt::NoPen);
-    painter.setBrush(palette().color(QPalette::Background).darker(107));
-    const int r = 3;
-    painter.drawRoundedRect(m_container->geometry().adjusted(r , r, -r, -r), r * 2, r * 2);
+
+    QRect rect = m_container->geometry();
+    if (creatorTheme()->flag(Theme::FlatToolBars)) {
+        painter.fillRect(rect, creatorTheme()->color(Theme::EditorPlaceholderColor));
+    } else {
+        painter.setRenderHint(QPainter::Antialiasing, true);
+        painter.setPen(Qt::NoPen);
+        painter.setBrush(creatorTheme()->color(Theme::EditorPlaceholderColor));
+        const int r = 3;
+        painter.drawRoundedRect(rect.adjusted(r , r, -r, -r), r * 2, r * 2);
+    }
 }
 
 void EditorView::mousePressEvent(QMouseEvent *e)
@@ -250,7 +308,7 @@ void EditorView::mousePressEvent(QMouseEvent *e)
 
 void EditorView::focusInEvent(QFocusEvent *)
 {
-    EditorManager::setCurrentView(this);
+    EditorManagerPrivate::setCurrentView(this);
 }
 
 void EditorView::addEditor(IEditor *editor)
@@ -286,49 +344,70 @@ void EditorView::removeEditor(IEditor *editor)
 
     m_container->removeWidget(editor->widget());
     m_widgetEditorMap.remove(editor->widget());
-    editor->widget()->setParent(0);
+    editor->widget()->setParent(nullptr);
     m_toolBar->removeToolbarForEditor(editor);
 
     if (wasCurrent)
-        setCurrentEditor(m_editors.count() ? m_editors.last() : 0);
+        setCurrentEditor(!m_editors.isEmpty() ? m_editors.last() : nullptr);
 }
 
 IEditor *EditorView::currentEditor() const
 {
-    if (m_editors.count() > 0)
+    if (!m_editors.isEmpty())
         return m_widgetEditorMap.value(m_container->currentWidget());
-    return 0;
+    return nullptr;
 }
 
 void EditorView::listSelectionActivated(int index)
 {
-    EditorManager::activateEditorForEntry(
-                this, EditorManager::documentModel()->documentAtRow(index));
+    EditorManagerPrivate::activateEditorForEntry(this, DocumentModel::entryAtRow(index));
+}
+
+void EditorView::fillListContextMenu(QMenu *menu)
+{
+    IEditor *editor = currentEditor();
+    DocumentModel::Entry *entry = editor ? DocumentModel::entryForDocument(editor->document())
+                                         : nullptr;
+    EditorManager::addSaveAndCloseEditorActions(menu, entry, editor);
+    menu->addSeparator();
+    EditorManager::addNativeDirAndOpenWithActions(menu, entry);
 }
 
 void EditorView::splitHorizontally()
 {
     if (m_parentSplitterOrView)
         m_parentSplitterOrView->split(Qt::Vertical);
-    EditorManager::updateActions();
+    EditorManagerPrivate::updateActions();
 }
 
 void EditorView::splitVertically()
 {
     if (m_parentSplitterOrView)
         m_parentSplitterOrView->split(Qt::Horizontal);
-    EditorManager::updateActions();
+    EditorManagerPrivate::updateActions();
 }
 
 void EditorView::splitNewWindow()
 {
-    EditorManager::splitNewWindow(this);
+    EditorManagerPrivate::splitNewWindow(this);
 }
 
 void EditorView::closeSplit()
 {
-    EditorManager::closeView(this);
-    EditorManager::updateActions();
+    EditorManagerPrivate::closeView(this);
+    EditorManagerPrivate::updateActions();
+}
+
+void EditorView::openDroppedFiles(const QList<DropSupport::FileSpec> &files)
+{
+    const int count = files.size();
+    for (int i = 0; i < count; ++i) {
+        const DropSupport::FileSpec spec = files.at(i);
+        EditorManagerPrivate::openEditorAt(this, spec.filePath, spec.line, spec.column, Id(),
+                                  i < count - 1 ? EditorManager::DoNotChangeCurrentEditor
+                                                  | EditorManager::DoNotMakeVisible
+                                                : EditorManager::NoFlags);
+    }
 }
 
 void EditorView::setParentSplitterOrView(SplitterOrView *splitterOrView)
@@ -340,9 +419,10 @@ void EditorView::setCurrentEditor(IEditor *editor)
 {
     if (!editor || m_container->indexOf(editor->widget()) == -1) {
         QTC_CHECK(!editor);
-        m_toolBar->setCurrentEditor(0);
-        m_infoBarDisplay->setInfoBar(0);
+        m_toolBar->setCurrentEditor(nullptr);
+        m_infoBarDisplay->setInfoBar(nullptr);
         m_container->setCurrentIndex(0);
+        emit currentEditorChanged(nullptr);
         return;
     }
 
@@ -357,6 +437,7 @@ void EditorView::setCurrentEditor(IEditor *editor)
     updateEditorHistory(editor);
 
     m_infoBarDisplay->setInfoBar(editor->document()->infoBar());
+    emit currentEditorChanged(editor);
 }
 
 int EditorView::editorCount() const
@@ -374,7 +455,7 @@ IEditor *EditorView::editorForDocument(const IDocument *document) const
     foreach (IEditor *editor, m_editors)
         if (editor->document() == document)
             return editor;
-    return 0;
+    return nullptr;
 }
 
 void EditorView::updateEditorHistory(IEditor *editor)
@@ -382,13 +463,9 @@ void EditorView::updateEditorHistory(IEditor *editor)
     updateEditorHistory(editor, m_editorHistory);
 }
 
-void EditorView::addCurrentPositionToNavigationHistory(IEditor *editor, const QByteArray &saveState)
+void EditorView::addCurrentPositionToNavigationHistory(const QByteArray &saveState)
 {
-    if (editor && editor != currentEditor())
-        return; // we only save editor sate for the current editor, when the user interacts
-
-    if (!editor)
-        editor = currentEditor();
+    IEditor *editor = currentEditor();
     if (!editor)
         return;
     IDocument *document = editor->document();
@@ -404,8 +481,8 @@ void EditorView::addCurrentPositionToNavigationHistory(IEditor *editor, const QB
 
     EditLocation location;
     location.document = document;
-    location.fileName = document->filePath();
-    location.id = editor->id();
+    location.fileName = document->filePath().toString();
+    location.id = document->id();
     location.state = QVariant(state);
     m_currentNavigationHistoryPosition = qMin(m_currentNavigationHistoryPosition, m_navigationHistory.size()); // paranoia
     m_navigationHistory.insert(m_currentNavigationHistoryPosition, location);
@@ -459,24 +536,35 @@ void EditorView::updateCurrentPositionInNavigationHistory()
         location = &m_navigationHistory[m_navigationHistory.size()-1];
     }
     location->document = document;
-    location->fileName = document->filePath();
-    location->id = editor->id();
+    location->fileName = document->filePath().toString();
+    location->id = document->id();
     location->state = QVariant(editor->saveState());
 }
 
+namespace {
+static inline bool fileNameWasRemoved(const QString &fileName)
+{
+    return !fileName.isEmpty() && !QFileInfo::exists(fileName);
+}
+} // End of anonymous namespace
+
 void EditorView::goBackInNavigationHistory()
 {
     updateCurrentPositionInNavigationHistory();
     while (m_currentNavigationHistoryPosition > 0) {
         --m_currentNavigationHistoryPosition;
         EditLocation location = m_navigationHistory.at(m_currentNavigationHistoryPosition);
-        IEditor *editor = 0;
+        IEditor *editor = nullptr;
         if (location.document) {
-            editor = EditorManager::activateEditorForDocument(this, location.document,
+            editor = EditorManagerPrivate::activateEditorForDocument(this, location.document,
                                         EditorManager::IgnoreNavigationHistory);
         }
         if (!editor) {
-            editor = EditorManager::openEditor(this, location.fileName, location.id,
+            if (fileNameWasRemoved(location.fileName)) {
+                m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
+                continue;
+            }
+            editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
                                     EditorManager::IgnoreNavigationHistory);
             if (!editor) {
                 m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
@@ -495,68 +583,112 @@ void EditorView::goForwardInNavigationHistory()
     if (m_currentNavigationHistoryPosition >= m_navigationHistory.size()-1)
         return;
     ++m_currentNavigationHistoryPosition;
-    EditLocation location = m_navigationHistory.at(m_currentNavigationHistoryPosition);
-    IEditor *editor = 0;
-    if (location.document) {
-        editor = EditorManager::activateEditorForDocument(this, location.document,
-                                    EditorManager::IgnoreNavigationHistory);
-    }
-    if (!editor) {
-        editor = EditorManager::openEditor(this, location.fileName, location.id, EditorManager::IgnoreNavigationHistory);
-        if (!editor) {
-            //TODO
-            qDebug() << Q_FUNC_INFO << "can't open file" << location.fileName;
-            return;
+    while (m_currentNavigationHistoryPosition < m_navigationHistory.size()) {
+        IEditor *editor = nullptr;
+        EditLocation location = m_navigationHistory.at(m_currentNavigationHistoryPosition);
+        if (location.document) {
+            editor = EditorManagerPrivate::activateEditorForDocument(this, location.document,
+                                                                     EditorManager::IgnoreNavigationHistory);
         }
+        if (!editor) {
+            if (fileNameWasRemoved(location.fileName)) {
+                m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
+                continue;
+            }
+            editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
+                                                      EditorManager::IgnoreNavigationHistory);
+            if (!editor) {
+                m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
+                continue;
+            }
+        }
+        editor->restoreState(location.state.toByteArray());
+        break;
     }
-    editor->restoreState(location.state.toByteArray());
+    if (m_currentNavigationHistoryPosition >= m_navigationHistory.size())
+        m_currentNavigationHistoryPosition = qMax<int>(m_navigationHistory.size() - 1, 0);
     updateNavigatorActions();
 }
 
+void EditorView::goToEditLocation(const EditLocation &location)
+{
+    IEditor *editor = nullptr;
 
-SplitterOrView::SplitterOrView(Core::IEditor *editor)
+    if (location.document) {
+        editor = EditorManagerPrivate::activateEditorForDocument(this, location.document,
+                                                                 EditorManager::IgnoreNavigationHistory);
+    }
+
+    if (!editor) {
+        if (fileNameWasRemoved(location.fileName))
+            return;
+
+        editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
+                                                  EditorManager::IgnoreNavigationHistory);
+    }
+
+    if (editor) {
+        editor->restoreState(location.state.toByteArray());
+    }
+}
+
+
+SplitterOrView::SplitterOrView(IEditor *editor)
 {
     m_layout = new QStackedLayout(this);
     m_layout->setSizeConstraint(QLayout::SetNoConstraint);
     m_view = new EditorView(this);
     if (editor)
         m_view->addEditor(editor);
-    m_splitter = 0;
+    m_splitter = nullptr;
     m_layout->addWidget(m_view);
 }
 
 SplitterOrView::SplitterOrView(EditorView *view)
 {
-    QTC_CHECK(view);
+    Q_ASSERT(view);
     m_layout = new QStackedLayout(this);
     m_layout->setSizeConstraint(QLayout::SetNoConstraint);
     m_view = view;
     m_view->setParentSplitterOrView(this);
-    m_splitter = 0;
+    m_splitter = nullptr;
     m_layout->addWidget(m_view);
 }
 
 SplitterOrView::~SplitterOrView()
 {
     delete m_layout;
-    m_layout = 0;
+    m_layout = nullptr;
     if (m_view)
-        EditorManager::emptyView(m_view);
+        EditorManagerPrivate::deleteEditors(EditorManagerPrivate::emptyView(m_view));
     delete m_view;
-    m_view = 0;
+    m_view = nullptr;
     delete m_splitter;
-    m_splitter = 0;
+    m_splitter = nullptr;
 }
 
 EditorView *SplitterOrView::findFirstView()
 {
     if (m_splitter) {
         for (int i = 0; i < m_splitter->count(); ++i) {
-            if (SplitterOrView *splitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(i)))
+            if (auto splitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(i)))
                 if (EditorView *result = splitterOrView->findFirstView())
                     return result;
         }
-        return 0;
+        return nullptr;
+    }
+    return m_view;
+}
+
+EditorView *SplitterOrView::findLastView()
+{
+    if (m_splitter) {
+        for (int i = m_splitter->count() - 1; 0 < i; --i) {
+            if (auto splitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(i)))
+                if (EditorView *result = splitterOrView->findLastView())
+                    return result;
+        }
+        return nullptr;
     }
     return m_view;
 }
@@ -565,13 +697,13 @@ SplitterOrView *SplitterOrView::findParentSplitter() const
 {
     QWidget *w = parentWidget();
     while (w) {
-        if (SplitterOrView *splitter = qobject_cast<SplitterOrView *>(w)) {
+        if (auto splitter = qobject_cast<SplitterOrView *>(w)) {
             QTC_CHECK(splitter->splitter());
             return splitter;
         }
         w = w->parentWidget();
     }
-    return 0;
+    return nullptr;
 }
 
 QSize SplitterOrView::minimumSizeHint() const
@@ -586,7 +718,7 @@ QSplitter *SplitterOrView::takeSplitter()
     QSplitter *oldSplitter = m_splitter;
     if (m_splitter)
         m_layout->removeWidget(m_splitter);
-    m_splitter = 0;
+    m_splitter = nullptr;
     return oldSplitter;
 }
 
@@ -596,27 +728,29 @@ EditorView *SplitterOrView::takeView()
     if (m_view) {
         // the focus update that is triggered by removing should already have 0 parent
         // so we do that first
-        m_view->setParentSplitterOrView(0);
+        m_view->setParentSplitterOrView(nullptr);
         m_layout->removeWidget(m_view);
     }
-    m_view = 0;
+    m_view = nullptr;
     return oldView;
 }
 
-void SplitterOrView::split(Qt::Orientation orientation)
+void SplitterOrView::split(Qt::Orientation orientation, bool activateView)
 {
-    Q_ASSERT(m_view && m_splitter == 0);
+    Q_ASSERT(m_view && m_splitter == nullptr);
     m_splitter = new MiniSplitter(this);
     m_splitter->setOrientation(orientation);
     m_layout->addWidget(m_splitter);
     m_layout->removeWidget(m_view);
     EditorView *editorView = m_view;
-    m_view = 0;
+    editorView->setCloseSplitEnabled(true); // might have been disabled for root view
+    m_view = nullptr;
     IEditor *e = editorView->currentEditor();
+    const QByteArray state = e ? e->saveState() : QByteArray();
 
-    SplitterOrView *view = 0;
-    SplitterOrView *otherView = 0;
-    IEditor *duplicate = e && e->duplicateSupported() ? EditorManager::duplicateEditor(e) : 0;
+    SplitterOrView *view = nullptr;
+    SplitterOrView *otherView = nullptr;
+    IEditor *duplicate = e && e->duplicateSupported() ? EditorManagerPrivate::duplicateEditor(e) : nullptr;
     m_splitter->addWidget((view = new SplitterOrView(duplicate)));
     m_splitter->addWidget((otherView = new SplitterOrView(editorView)));
 
@@ -626,17 +760,23 @@ void SplitterOrView::split(Qt::Orientation orientation)
     view->view()->setCurrentEditor(duplicate);
 
     if (orientation == Qt::Horizontal) {
-        view->view()->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_LEFT)));
-        otherView->view()->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_RIGHT)));
+        view->view()->setCloseSplitIcon(Utils::Icons::CLOSE_SPLIT_LEFT.icon());
+        otherView->view()->setCloseSplitIcon(Utils::Icons::CLOSE_SPLIT_RIGHT.icon());
     } else {
-        view->view()->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_TOP)));
-        otherView->view()->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_BOTTOM)));
+        view->view()->setCloseSplitIcon(Utils::Icons::CLOSE_SPLIT_TOP.icon());
+        otherView->view()->setCloseSplitIcon(Utils::Icons::CLOSE_SPLIT_BOTTOM.icon());
     }
 
+    // restore old state, possibly adapted to the new layout (the editors can e.g. make sure that
+    // a previously visible text cursor stays visible)
+    if (duplicate)
+        duplicate->restoreState(state);
     if (e)
-        EditorManager::activateEditor(otherView->view(), e);
-    else
-        EditorManager::setCurrentView(otherView->view());
+        e->restoreState(state);
+
+    if (activateView)
+        EditorManagerPrivate::activateView(otherView->view());
+    emit splitStateChanged();
 }
 
 void SplitterOrView::unsplitAll()
@@ -651,7 +791,7 @@ void SplitterOrView::unsplitAll()
         }
     }
 
-    EditorView *currentView = EditorManager::currentEditorView();
+    EditorView *currentView = EditorManagerPrivate::currentEditorView();
     if (currentView) {
         currentView->parentSplitterOrView()->takeView();
         currentView->setParentSplitterOrView(this);
@@ -660,11 +800,11 @@ void SplitterOrView::unsplitAll()
     }
     m_splitter->hide();
     m_layout->removeWidget(m_splitter); // workaround Qt bug
-    unsplitAll_helper();
+    const QList<IEditor *> editorsToDelete = unsplitAll_helper();
     m_view = currentView;
     m_layout->addWidget(m_view);
     delete m_splitter;
-    m_splitter = 0;
+    m_splitter = nullptr;
 
     // restore some focus
     if (hadFocus) {
@@ -673,18 +813,27 @@ void SplitterOrView::unsplitAll()
         else
             m_view->setFocus();
     }
+    EditorManagerPrivate::deleteEditors(editorsToDelete);
+    emit splitStateChanged();
 }
 
-void SplitterOrView::unsplitAll_helper()
+/*!
+    Recursively empties all views.
+    Returns the editors to delete with EditorManagerPrivate::deleteEditors.
+    \internal
+*/
+const QList<IEditor *> SplitterOrView::unsplitAll_helper()
 {
     if (m_view)
-        EditorManager::emptyView(m_view);
+        return EditorManagerPrivate::emptyView(m_view);
+    QList<IEditor *> editorsToDelete;
     if (m_splitter) {
         for (int i = 0; i < m_splitter->count(); ++i) {
-            if (SplitterOrView *splitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(i)))
-                splitterOrView->unsplitAll_helper();
+            if (auto splitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(i)))
+                editorsToDelete.append(splitterOrView->unsplitAll_helper());
         }
     }
+    return editorsToDelete;
 }
 
 void SplitterOrView::unsplit()
@@ -693,12 +842,12 @@ void SplitterOrView::unsplit()
         return;
 
     Q_ASSERT(m_splitter->count() == 1);
-    SplitterOrView *childSplitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(0));
+    auto childSplitterOrView = qobject_cast<SplitterOrView*>(m_splitter->widget(0));
     QSplitter *oldSplitter = m_splitter;
-    m_splitter = 0;
-
+    m_splitter = nullptr;
+    QList<IEditor *> editorsToDelete;
     if (childSplitterOrView->isSplitter()) {
-        Q_ASSERT(childSplitterOrView->view() == 0);
+        Q_ASSERT(childSplitterOrView->view() == nullptr);
         m_splitter = childSplitterOrView->takeSplitter();
         m_layout->addWidget(m_splitter);
         m_layout->setCurrentWidget(m_splitter);
@@ -712,30 +861,32 @@ void SplitterOrView::unsplit()
                 m_view->addEditor(e);
                 m_view->setCurrentEditor(e);
             }
-            EditorManager::emptyView(childView);
+            editorsToDelete = EditorManagerPrivate::emptyView(childView);
         } else {
             m_view = childSplitterOrView->takeView();
             m_view->setParentSplitterOrView(this);
             m_layout->addWidget(m_view);
-            QSplitter *parentSplitter = qobject_cast<QSplitter *>(parentWidget());
+            auto parentSplitter = qobject_cast<QSplitter *>(parentWidget());
             if (parentSplitter) { // not the toplevel splitterOrView
-                if (parentSplitter->orientation() == Qt::Horizontal) {
-                    if (parentSplitter->widget(0) == this)
-                        m_view->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_LEFT)));
-                    else
-                        m_view->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_RIGHT)));
-                } else {
-                    if (parentSplitter->widget(0) == this)
-                        m_view->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_TOP)));
-                    else
-                        m_view->setCloseSplitIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_SPLIT_BOTTOM)));
-                }
+                if (parentSplitter->orientation() == Qt::Horizontal)
+                    m_view->setCloseSplitIcon(parentSplitter->widget(0) == this ?
+                                                  Utils::Icons::CLOSE_SPLIT_LEFT.icon()
+                                                : Utils::Icons::CLOSE_SPLIT_RIGHT.icon());
+                else
+                    m_view->setCloseSplitIcon(parentSplitter->widget(0) == this ?
+                                                  Utils::Icons::CLOSE_SPLIT_TOP.icon()
+                                                : Utils::Icons::CLOSE_SPLIT_BOTTOM.icon());
             }
         }
         m_layout->setCurrentWidget(m_view);
     }
     delete oldSplitter;
-    EditorManager::setCurrentView(findFirstView());
+    if (EditorView *newCurrent = findFirstView())
+        EditorManagerPrivate::activateView(newCurrent);
+    else
+        EditorManagerPrivate::setCurrentView(nullptr);
+    EditorManagerPrivate::deleteEditors(editorsToDelete);
+    emit splitStateChanged();
 }
 
 
@@ -756,7 +907,7 @@ QByteArray SplitterOrView::saveState() const
         // don't save state of temporary or ad-hoc editors
         if (e && (e->document()->isTemporary() || e->document()->filePath().isEmpty())) {
             // look for another editor that is more suited
-            e = 0;
+            e = nullptr;
             foreach (IEditor *otherEditor, editors()) {
                 if (!otherEditor->document()->isTemporary() && !otherEditor->document()->filePath().isEmpty()) {
                     e = otherEditor;
@@ -769,10 +920,14 @@ QByteArray SplitterOrView::saveState() const
             stream << QByteArray("empty");
         } else if (e == EditorManager::currentEditor()) {
             stream << QByteArray("currenteditor")
-                    << e->document()->filePath() << e->id().toString() << e->saveState();
+                   << e->document()->filePath().toString()
+                   << e->document()->id().toString()
+                   << e->saveState();
         } else {
             stream << QByteArray("editor")
-                    << e->document()->filePath() << e->id().toString() << e->saveState();
+                   << e->document()->filePath().toString()
+                   << e->document()->id().toString()
+                   << e->saveState();
         }
     }
     return bytes;
@@ -787,7 +942,7 @@ void SplitterOrView::restoreState(const QByteArray &state)
         qint32 orientation;
         QByteArray splitter, first, second;
         stream >> orientation >> splitter >> first >> second;
-        split((Qt::Orientation)orientation);
+        split((Qt::Orientation) orientation, false);
         m_splitter->restoreState(splitter);
         static_cast<SplitterOrView*>(m_splitter->widget(0))->restoreState(first);
         static_cast<SplitterOrView*>(m_splitter->widget(1))->restoreState(second);
@@ -798,20 +953,22 @@ void SplitterOrView::restoreState(const QByteArray &state)
         stream >> fileName >> id >> editorState;
         if (!QFile::exists(fileName))
             return;
-        IEditor *e = EditorManager::openEditor(view(), fileName, Id::fromString(id), Core::EditorManager::IgnoreNavigationHistory
-                                    | Core::EditorManager::DoNotChangeCurrentEditor);
+        IEditor *e = EditorManagerPrivate::openEditor(view(), fileName, Id::fromString(id),
+                                                      EditorManager::IgnoreNavigationHistory
+                                                      | EditorManager::DoNotChangeCurrentEditor);
 
         if (!e) {
-            DocumentModel::Entry *entry = EditorManager::documentModel()->firstRestoredDocument();
-            if (entry)
-                EditorManager::activateEditorForEntry(view(), entry, Core::EditorManager::IgnoreNavigationHistory
-                                    | Core::EditorManager::DoNotChangeCurrentEditor);
+            DocumentModel::Entry *entry = DocumentModelPrivate::firstSuspendedEntry();
+            if (entry) {
+                EditorManagerPrivate::activateEditorForEntry(view(), entry,
+                    EditorManager::IgnoreNavigationHistory | EditorManager::DoNotChangeCurrentEditor);
+            }
         }
 
         if (e) {
             e->restoreState(editorState);
             if (mode == "currenteditor")
-                EditorManager::setCurrentEditor(e);
+                EditorManagerPrivate::setCurrentEditor(e);
         }
     }
 }
