@@ -34,42 +34,17 @@
 #include <utils/qtcassert.h>
 #include <utils/styledbar.h>
 
-namespace {
-const int ICON_SIZE(64);
-
-const int ABOVE_HEADING_MARGIN(10);
-const int ABOVE_CONTENTS_MARGIN(4);
-const int BELOW_CONTENTS_MARGIN(16);
-const int PANEL_LEFT_MARGIN = 70;
-
-} // anonymous namespace
-///
-// OnePixelBlackLine
-///
-/// \brief The OnePixelBlackLine class
-
-using namespace ProjectExplorer;
 using namespace Utils;
 
+namespace ProjectExplorer {
 namespace {
-class OnePixelBlackLine : public QWidget
-{
-public:
-    OnePixelBlackLine(QWidget *parent)
-        : QWidget(parent)
-    {
-        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-        setMinimumHeight(1);
-        setMaximumHeight(1);
-    }
-    void paintEvent(QPaintEvent *e)
-    {
-        Q_UNUSED(e);
-        QPainter p(this);
-        QColor fillColor = creatorTheme()->color(Theme::PanelsWidgetSeparatorLineColor);
-        p.fillRect(contentsRect(), fillColor);
-    }
-};
+
+const int ICON_SIZE(64);
+
+const int ABOVE_HEADING_MARGIN = 10;
+const int ABOVE_CONTENTS_MARGIN = 4;
+const int BELOW_CONTENTS_MARGIN = 16;
+const int PANEL_LEFT_MARGIN = 70;
 
 class RootWidget : public QWidget
 {
@@ -77,7 +52,7 @@ public:
     RootWidget(QWidget *parent) : QWidget(parent) {
         setFocusPolicy(Qt::NoFocus);
     }
-    void paintEvent(QPaintEvent *);
+    void paintEvent(QPaintEvent *) override;
 };
 
 void RootWidget::paintEvent(QPaintEvent *e)
@@ -112,13 +87,6 @@ PanelsWidget::PanelsWidget(QWidget *parent) :
     // side of the screen.
     m_root->setMaximumWidth(900);
     m_root->setContentsMargins(0, 0, 40, 0);
-    QPalette pal;
-    QColor background = StyleHelper::mergedColors(
-                palette().window().color(), Qt::white, 85);
-    pal.setColor(QPalette::All, QPalette::Window, background.darker(102));
-    setPalette(pal);
-    pal.setColor(QPalette::All, QPalette::Window, background);
-    m_root->setPalette(pal);
 
     m_scroller = new QScrollArea(this);
     m_scroller->setWidget(m_root);
@@ -128,7 +96,7 @@ PanelsWidget::PanelsWidget(QWidget *parent) :
 
     // The layout holding the individual panels:
     auto topLayout = new QVBoxLayout(m_root);
-    topLayout->setMargin(0);
+    topLayout->setContentsMargins(0, 0, 0, 0);
     topLayout->setSpacing(0);
 
     m_layout = new QGridLayout;
@@ -147,10 +115,14 @@ PanelsWidget::PanelsWidget(QWidget *parent) :
     //layout->addWidget(new FindToolBarPlaceHolder(this));
 }
 
-PanelsWidget::~PanelsWidget()
+PanelsWidget::PanelsWidget(const QString &displayName, const QIcon &icon, QWidget *widget)
+    : PanelsWidget(nullptr)
 {
+    addPropertiesPanel(displayName, icon, widget);
 }
 
+PanelsWidget::~PanelsWidget() = default;
+
 /*
  * Add a widget with heading information into the grid
  * layout of the PanelsWidget.
@@ -179,14 +151,6 @@ void PanelsWidget::addPropertiesPanel(const QString &displayName, const QIcon &i
     // name:
     auto nameLabel = new QLabel(m_root);
     nameLabel->setText(displayName);
-    QPalette palette = nameLabel->palette();
-    for (int i = QPalette::Active; i < QPalette::NColorGroups; ++i ) {
-        // FIXME: theming
-        QColor foregroundColor = palette.color(QPalette::ColorGroup(i), QPalette::Foreground);
-        foregroundColor.setAlpha(110);
-        palette.setBrush(QPalette::ColorGroup(i), QPalette::Foreground, foregroundColor);
-    }
-    nameLabel->setPalette(palette);
     nameLabel->setContentsMargins(0, ABOVE_HEADING_MARGIN, 0, 0);
     QFont f = nameLabel->font();
     f.setBold(true);
@@ -196,7 +160,10 @@ void PanelsWidget::addPropertiesPanel(const QString &displayName, const QIcon &i
 
     // line:
     const int lineRow = headerRow + 1;
-    auto line = new OnePixelBlackLine(m_root);
+    auto line = new QFrame(m_root);
+    line->setFrameShape(QFrame::HLine);
+    line->setForegroundRole(QPalette::Midlight);
+    line->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     m_layout->addWidget(line, lineRow, 1, 1, -1, Qt::AlignTop);
 
     // add the widget:
@@ -208,3 +175,5 @@ void PanelsWidget::addPropertiesPanel(const QString &displayName, const QIcon &i
     widget->setParent(m_root);
     m_layout->addWidget(widget, widgetRow, 0, 1, 2);
 }
+
+} // ProjectExplorer
