@@ -27,12 +27,10 @@
 
 #include "projectexplorer_export.h"
 
-#include <coreplugin/id.h>
+#include "toolchain.h"
 
 #include <utils/fileutils.h>
 
-#include <QVariant>
-
 namespace ProjectExplorer {
 
 class BuildInfo;
@@ -44,19 +42,20 @@ class ToolChain;
 // Documentation inside.
 class PROJECTEXPLORER_EXPORT ProjectImporter : public QObject
 {
+    Q_OBJECT
 public:
     struct ToolChainData {
         QList<ToolChain *> tcs;
         bool areTemporary = false;
     };
 
-    ProjectImporter(const Utils::FileName &path);
-    virtual ~ProjectImporter();
+    ProjectImporter(const Utils::FilePath &path);
+    ~ProjectImporter() override;
 
-    const Utils::FileName projectFilePath() const { return m_projectPath; }
-    const Utils::FileName projectDirectory() const { return m_projectPath.parentDir(); }
+    const Utils::FilePath projectFilePath() const { return m_projectPath; }
+    const Utils::FilePath projectDirectory() const { return m_projectPath.parentDir(); }
 
-    virtual QList<BuildInfo *> import(const Utils::FileName &importPath, bool silent = false);
+    virtual const QList<BuildInfo> import(const Utils::FilePath &importPath, bool silent = false);
     virtual QStringList importCandidates() = 0;
     virtual Target *preferredTarget(const QList<Target *> &possibleTargets);
 
@@ -87,13 +86,13 @@ protected:
     };
 
     // importPath is an existing directory at this point!
-    virtual QList<void *> examineDirectory(const Utils::FileName &importPath) const = 0;
+    virtual QList<void *> examineDirectory(const Utils::FilePath &importPath) const = 0;
     // will get one of the results from examineDirectory
     virtual bool matchKit(void *directoryData, const Kit *k) const = 0;
     // will get one of the results from examineDirectory
     virtual Kit *createKit(void *directoryData) const = 0;
     // will get one of the results from examineDirectory
-    virtual QList<BuildInfo *> buildInfoListForKit(const Kit *k, void *directoryData) const = 0;
+    virtual const QList<BuildInfo> buildInfoList(void *directoryData) const = 0;
 
     virtual void deleteDirectoryData(void *directoryData) const = 0;
 
@@ -103,13 +102,13 @@ protected:
     // Handle temporary additions to Kits (Qt Versions, ToolChains, etc.)
     using CleanupFunction = std::function<void(Kit *, const QVariantList &)>;
     using PersistFunction = std::function<void(Kit *, const QVariantList &)>;
-    void useTemporaryKitInformation(Core::Id id,
+    void useTemporaryKitAspect(Core::Id id,
                                     CleanupFunction cleanup, PersistFunction persist);
     void addTemporaryData(Core::Id id, const QVariant &cleanupData, Kit *k) const;
     // Does *any* kit feature the requested data yet?
     bool hasKitWithTemporaryData(Core::Id id, const QVariant &data) const;
 
-    ToolChainData findOrCreateToolChains(const Utils::FileName &toolChainPath, const Core::Id &language) const;
+    ToolChainData findOrCreateToolChains(const ToolChainDescription &tcd) const;
 
 private:
     void markKitAsTemporary(Kit *k) const;
@@ -118,7 +117,7 @@ private:
     void cleanupTemporaryToolChains(ProjectExplorer::Kit *k, const QVariantList &vl);
     void persistTemporaryToolChains(ProjectExplorer::Kit *k, const QVariantList &vl);
 
-    const Utils::FileName m_projectPath;
+    const Utils::FilePath m_projectPath;
     mutable bool m_isUpdating = false;
 
     class TemporaryInformationHandler {
