@@ -47,7 +47,6 @@ enum {
 namespace QmlJSEditor {
 namespace Internal {
 
-
 QmlJSOutlineFilterModel::QmlJSOutlineFilterModel(QObject *parent) :
     QSortFilterProxyModel(parent)
 {
@@ -94,22 +93,19 @@ void QmlJSOutlineFilterModel::setFilterBindings(bool filterBindings)
     invalidateFilter();
 }
 
-QmlJSOutlineWidget::QmlJSOutlineWidget(QWidget *parent) :
-    TextEditor::IOutlineWidget(parent),
-    m_treeView(new QmlJSOutlineTreeView(this)),
-    m_filterModel(new QmlJSOutlineFilterModel(this)),
-    m_editor(0),
-    m_enableCursorSync(true),
-    m_blockCursorSync(false)
+QmlJSOutlineWidget::QmlJSOutlineWidget(QWidget *parent)
+    : TextEditor::IOutlineWidget(parent)
+    , m_treeView(new QmlJSOutlineTreeView(this))
+    , m_filterModel(new QmlJSOutlineFilterModel(this))
 {
     m_filterModel->setFilterBindings(false);
 
     m_treeView->setModel(m_filterModel);
     setFocusProxy(m_treeView);
 
-    QVBoxLayout *layout = new QVBoxLayout;
+    auto layout = new QVBoxLayout;
 
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
     layout->addWidget(Core::ItemViewFind::createSearchableWrapper(m_treeView));
 
@@ -127,7 +123,15 @@ void QmlJSOutlineWidget::setEditor(QmlJSEditorWidget *editor)
     m_editor = editor;
 
     m_filterModel->setSourceModel(m_editor->qmlJsEditorDocument()->outlineModel());
-    modelUpdated();
+    m_treeView->expandAll();
+    connect(m_editor->qmlJsEditorDocument()->outlineModel(), &QAbstractItemModel::modelAboutToBeReset, m_treeView, [this]() {
+        if (m_treeView->selectionModel())
+            m_treeView->selectionModel()->blockSignals(true);
+    });
+    connect(m_editor->qmlJsEditorDocument()->outlineModel(), &QAbstractItemModel::modelReset, m_treeView, [this]() {
+        if (m_treeView->selectionModel())
+            m_treeView->selectionModel()->blockSignals(false);
+    });
 
     connect(m_treeView->selectionModel(), &QItemSelectionModel::selectionChanged,
             this, &QmlJSOutlineWidget::updateSelectionInText);
@@ -137,22 +141,21 @@ void QmlJSOutlineWidget::setEditor(QmlJSEditorWidget *editor)
 
     connect(m_editor, &QmlJSEditorWidget::outlineModelIndexChanged,
             this, &QmlJSOutlineWidget::updateSelectionInTree);
-    connect(m_editor->qmlJsEditorDocument()->outlineModel(), &QmlOutlineModel::updated,
-            this, &QmlJSOutlineWidget::modelUpdated);
+    connect(m_editor->qmlJsEditorDocument()->outlineModel(), &QmlOutlineModel::updated, this, [this] () {
+        m_treeView->expandAll();
+        m_editor->updateOutlineIndexNow();
+    });
 }
 
 QList<QAction*> QmlJSOutlineWidget::filterMenuActions() const
 {
-    QList<QAction*> list;
-    list.append(m_showBindingsAction);
-    return list;
+    return {m_showBindingsAction};
 }
 
 void QmlJSOutlineWidget::setCursorSynchronization(bool syncWithCursor)
 {
     m_enableCursorSync = syncWithCursor;
-    if (m_enableCursorSync)
-        updateSelectionInTree(m_editor->outlineModelIndex());
+    m_editor->updateOutlineIndexNow();
 }
 
 void QmlJSOutlineWidget::restoreSettings(const QVariantMap &map)
@@ -163,14 +166,7 @@ void QmlJSOutlineWidget::restoreSettings(const QVariantMap &map)
 
 QVariantMap QmlJSOutlineWidget::settings() const
 {
-    QVariantMap map;
-    map.insert(QLatin1String("QmlJSOutline.ShowBindings"), m_showBindingsAction->isChecked());
-    return map;
-}
-
-void QmlJSOutlineWidget::modelUpdated()
-{
-    m_treeView->expandAll();
+    return {{QLatin1String("QmlJSOutline.ShowBindings"), m_showBindingsAction->isChecked()}};
 }
 
 void QmlJSOutlineWidget::updateSelectionInTree(const QModelIndex &index)
@@ -206,26 +202,33 @@ void QmlJSOutlineWidget::updateSelectionInText(const QItemSelection &selection)
 
 void QmlJSOutlineWidget::updateTextCursor(const QModelIndex &index)
 {
-    QModelIndex sourceIndex = m_filterModel->mapToSource(index);
-    AST::SourceLocation location
-            = m_editor->qmlJsEditorDocument()->outlineModel()->sourceLocation(sourceIndex);
+    const auto update = [this](const QModelIndex &index) {
+        if (!m_editor->isOutlineCursorChangesBlocked()) {
+            QModelIndex sourceIndex = m_filterModel->mapToSource(index);
 
-    if (!location.isValid())
-        return;
+            SourceLocation location
+                    = m_editor->qmlJsEditorDocument()->outlineModel()->sourceLocation(sourceIndex);
 
-    const QTextBlock lastBlock = m_editor->document()->lastBlock();
-    const uint textLength = lastBlock.position() + lastBlock.length();
-    if (location.offset >= textLength)
-        return;
+            if (!location.isValid())
+                return;
 
-    Core::EditorManager::cutForwardNavigationHistory();
-    Core::EditorManager::addCurrentPositionToNavigationHistory();
+            const QTextBlock lastBlock = m_editor->document()->lastBlock();
+            const uint textLength = lastBlock.position() + lastBlock.length();
+            if (location.offset >= textLength)
+                return;
 
-    QTextCursor textCursor = m_editor->textCursor();
+            Core::EditorManager::cutForwardNavigationHistory();
+            Core::EditorManager::addCurrentPositionToNavigationHistory();
+
+            QTextCursor textCursor = m_editor->textCursor();
+
+            textCursor.setPosition(location.offset);
+            m_editor->setTextCursor(textCursor);
+            m_editor->centerCursor();
+        }
+    };
     m_blockCursorSync = true;
-    textCursor.setPosition(location.offset);
-    m_editor->setTextCursor(textCursor);
-    m_editor->centerCursor();
+    update(index);
     m_blockCursorSync = false;
 }
 
@@ -237,8 +240,8 @@ void QmlJSOutlineWidget::focusEditor()
 void QmlJSOutlineWidget::setShowBindings(bool showBindings)
 {
     m_filterModel->setFilterBindings(!showBindings);
-    modelUpdated();
-    updateSelectionInTree(m_editor->outlineModelIndex());
+    m_treeView->expandAll();
+    m_editor->updateOutlineIndexNow();
 }
 
 bool QmlJSOutlineWidget::syncCursor()
@@ -255,10 +258,10 @@ bool QmlJSOutlineWidgetFactory::supportsEditor(Core::IEditor *editor) const
 
 TextEditor::IOutlineWidget *QmlJSOutlineWidgetFactory::createWidget(Core::IEditor *editor)
 {
-    QmlJSOutlineWidget *widget = new QmlJSOutlineWidget;
+    auto widget = new QmlJSOutlineWidget;
 
-    QmlJSEditor *qmlJSEditable = qobject_cast<QmlJSEditor*>(editor);
-    QmlJSEditorWidget *qmlJSEditor = qobject_cast<QmlJSEditorWidget*>(qmlJSEditable->widget());
+    auto qmlJSEditable = qobject_cast<const QmlJSEditor*>(editor);
+    auto qmlJSEditor = qobject_cast<QmlJSEditorWidget*>(qmlJSEditable->widget());
     Q_ASSERT(qmlJSEditor);
 
     widget->setEditor(qmlJSEditor);
