@@ -29,10 +29,10 @@
 #include "snippet.h"
 #include "snippetscollection.h"
 #include "snippetssettings.h"
-#include "textdocument.h"
 #include "ui_snippetssettingspage.h"
 
 #include <coreplugin/icore.h>
+#include <texteditor/textdocument.h>
 #include <texteditor/texteditorconstants.h>
 #include <texteditor/texteditorsettings.h>
 #include <extensionsystem/pluginmanager.h>
@@ -54,16 +54,16 @@ class SnippetsTableModel : public QAbstractTableModel
     Q_OBJECT
 public:
     SnippetsTableModel(QObject *parent);
-    virtual ~SnippetsTableModel() {}
+    ~SnippetsTableModel() override = default;
 
-    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
-    virtual Qt::ItemFlags flags(const QModelIndex &modelIndex) const;
-    virtual QVariant data(const QModelIndex &modelIndex, int role = Qt::DisplayRole) const;
-    virtual bool setData(const QModelIndex &modelIndex, const QVariant &value,
-                         int role = Qt::EditRole);
-    virtual QVariant headerData(int section, Qt::Orientation orientation,
-                                int role = Qt::DisplayRole) const;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+    Qt::ItemFlags flags(const QModelIndex &modelIndex) const override;
+    QVariant data(const QModelIndex &modelIndex, int role = Qt::DisplayRole) const override;
+    bool setData(const QModelIndex &modelIndex, const QVariant &value,
+                 int role = Qt::EditRole) override;
+    QVariant headerData(int section, Qt::Orientation orientation,
+                        int role = Qt::DisplayRole) const override;
 
     QList<QString> groupIds() const;
     void load(const QString &groupId);
@@ -131,7 +131,8 @@ bool SnippetsTableModel::setData(const QModelIndex &modelIndex, const QVariant &
         if (modelIndex.column() == 0) {
             const QString &s = value.toString();
             if (!isValidTrigger(s)) {
-                QMessageBox::critical(0, tr("Error"), tr("Not a valid trigger."));
+                QMessageBox::critical(Core::ICore::dialogParent(), tr("Error"),
+                                      tr("Not a valid trigger."));
                 if (snippet.trigger().isEmpty())
                     removeSnippet(modelIndex);
                 return false;
@@ -207,7 +208,8 @@ void SnippetsTableModel::revertBuitInSnippet(const QModelIndex &modelIndex)
 {
     const Snippet &snippet = m_collection->revertedSnippet(modelIndex.row(), m_activeGroupId);
     if (snippet.id().isEmpty()) {
-        QMessageBox::critical(0, tr("Error"), tr("Error reverting snippet."));
+        QMessageBox::critical(Core::ICore::dialogParent(), tr("Error"),
+                              tr("Error reverting snippet."));
         return;
     }
     replaceSnippet(snippet, modelIndex);
@@ -262,13 +264,12 @@ bool SnippetsTableModel::isValidTrigger(const QString &s)
 // SnippetsSettingsPagePrivate
 class SnippetsSettingsPagePrivate : public QObject
 {
-    Q_OBJECT
+    Q_DECLARE_TR_FUNCTIONS(TextEditor::Internal::SnippetsSettingsPage)
+
 public:
-    SnippetsSettingsPagePrivate(Core::Id id);
-    ~SnippetsSettingsPagePrivate() { delete m_model; }
+    SnippetsSettingsPagePrivate();
+    ~SnippetsSettingsPagePrivate() override { delete m_model; }
 
-    Core::Id id() const { return m_id; }
-    const QString &displayName() const { return m_displayName; }
     void configureUi(QWidget *parent);
 
     void apply();
@@ -297,8 +298,6 @@ private:
     bool settingsChanged() const;
     void writeSettings();
 
-    const Core::Id m_id;
-    const QString m_displayName;
     const QString m_settingsPrefix;
     SnippetsTableModel *m_model;
     bool m_snippetsCollectionChanged;
@@ -306,11 +305,9 @@ private:
     Ui::SnippetsSettingsPage m_ui;
 };
 
-SnippetsSettingsPagePrivate::SnippetsSettingsPagePrivate(Core::Id id) :
-    m_id(id),
-    m_displayName(tr("Snippets")),
+SnippetsSettingsPagePrivate::SnippetsSettingsPagePrivate() :
     m_settingsPrefix(QLatin1String("Text")),
-    m_model(new SnippetsTableModel(0)),
+    m_model(new SnippetsTableModel(nullptr)),
     m_snippetsCollectionChanged(false)
 {}
 
@@ -330,7 +327,7 @@ void SnippetsSettingsPagePrivate::configureUi(QWidget *w)
 
     for (const SnippetProvider &provider : SnippetProvider::snippetProviders()) {
         m_ui.groupCombo->addItem(provider.displayName(), provider.groupId());
-        SnippetEditorWidget *snippetEditor = new SnippetEditorWidget(w);
+        auto snippetEditor = new SnippetEditorWidget(w);
         SnippetProvider::decorateEditor(snippetEditor, provider.groupId());
         m_ui.snippetsEditorStack->insertWidget(m_ui.groupCombo->count() - 1, snippetEditor);
         connect(snippetEditor, &SnippetEditorWidget::snippetContentChanged,
@@ -362,7 +359,7 @@ void SnippetsSettingsPagePrivate::configureUi(QWidget *w)
     connect(m_model, &QAbstractItemModel::modelReset,
             this, &SnippetsSettingsPagePrivate::markSnippetsCollection);
 
-    connect(m_ui.groupCombo, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(m_ui.groupCombo, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &SnippetsSettingsPagePrivate::loadSnippetGroup);
     connect(m_ui.addButton, &QAbstractButton::clicked,
             this, &SnippetsSettingsPagePrivate::addSnippet);
@@ -391,11 +388,12 @@ void SnippetsSettingsPagePrivate::apply()
 
     if (m_snippetsCollectionChanged) {
         QString errorString;
-        if (SnippetsCollection::instance()->synchronize(&errorString))
+        if (SnippetsCollection::instance()->synchronize(&errorString)) {
             m_snippetsCollectionChanged = false;
-        else
-            QMessageBox::critical(Core::ICore::mainWindow(),
-                    tr("Error While Saving Snippet Collection"), errorString);
+        } else {
+            QMessageBox::critical(Core::ICore::dialogParent(),
+                                  tr("Error While Saving Snippet Collection"), errorString);
+        }
     }
 }
 
@@ -406,7 +404,7 @@ void SnippetsSettingsPagePrivate::finish()
         m_snippetsCollectionChanged = false;
     }
 
-    disconnect(TextEditorSettings::instance(), 0, this, 0);
+    disconnect(TextEditorSettings::instance(), nullptr, this, nullptr);
 }
 
 void SnippetsSettingsPagePrivate::loadSettings()
@@ -466,7 +464,7 @@ void SnippetsSettingsPagePrivate::removeSnippet()
 {
     const QModelIndex &modelIndex = m_ui.snippetsTable->selectionModel()->currentIndex();
     if (!modelIndex.isValid()) {
-        QMessageBox::critical(0, tr("Error"), tr("No snippet selected."));
+        QMessageBox::critical(Core::ICore::dialogParent(), tr("Error"), tr("No snippet selected."));
         return;
     }
     m_model->removeSnippet(modelIndex);
@@ -545,12 +543,14 @@ void SnippetsSettingsPagePrivate::decorateEditors(const TextEditor::FontSettings
 }
 
 // SnippetsSettingsPage
-SnippetsSettingsPage::SnippetsSettingsPage(Core::Id id, QObject *parent) :
-    TextEditorOptionsPage(parent),
-    d(new SnippetsSettingsPagePrivate(id))
+SnippetsSettingsPage::SnippetsSettingsPage()
+    : d(new SnippetsSettingsPagePrivate)
 {
-    setId(d->id());
-    setDisplayName(d->displayName());
+    setId(Constants::TEXT_EDITOR_SNIPPETS_SETTINGS);
+    setDisplayName(SnippetsSettingsPagePrivate::tr("Snippets"));
+    setCategory(TextEditor::Constants::TEXT_EDITOR_SETTINGS_CATEGORY);
+    setDisplayCategory(QCoreApplication::translate("TextEditor", "Text Editor"));
+    setCategoryIconPath(TextEditor::Constants::TEXT_EDITOR_SETTINGS_CATEGORY_ICON_PATH);
 }
 
 SnippetsSettingsPage::~SnippetsSettingsPage()
